#!/usr/bin/env python3
from __init__ import *

class Cups(BaseScript):
    def __init__(self, params):
        service_name = 'ipp'
        service_config_dir = '/etc/cups'
        service_owner = 'root'
        available_modes = [MODE_PAM, MODE_ALD]
        params.update({
            MESSAGES: {
                CONFIGURE:
                    {
                        MODE_ALD: 'Администратору дополнительно требуется внести пользователя, '
                                  'от имени которого проводится маркировка, в группы \'lpmac\' '
                                  'и \'print_admins\''
                    }
            },
            INCOMPATIBLE_MODES: [(MODE_PAM, MODE_ALD)],
            KEYTAB_DIRECTORY: '/etc',
            INIT_D_SERVICE_NAME: 'cups'
        })
        self.__cups_d_conf = '%s/cupsd.conf' % service_config_dir
        self.__cups_files_conf = '%s/cups-files.conf' % service_config_dir
        self.__cupd_d_listen_data_pam = ['DefaultAuthType Basic',
                                         'ServerAlias *',
                                         'DefaultPolicy authenticated',
                                         'ServerName %s' % BaseUtils.hostname(),
                                         'Listen 0.0.0.0:631',
                                         'Listen /var/run/cups/cups.sock'
                                         ]
        self.__cupd_d_listen_data_ald = ['ServerAlias *',
                                         'DefaultPolicy authenticated',
                                         'Listen 0.0.0.0:631',
                                         'MarkerUser ipp',
                                         'ServerName %s' % BaseUtils.hostname(),
                                         'MacEnable On',
                                         'DefaultAuthType Negotiate'
                                         ]
        super().__init__(service_name,
                         service_owner,
                         service_config_dir=service_config_dir,
                         available_modes=available_modes,
                         params=params
                         )
        
    def __add_lpmac_group(self):
        """
        Добавляет локальную группу lpmac
        :return None
        """
        try:
            self.local_utils.exec_command('groupadd lpmac --gid 900 --system')
        except RbtConfigError:
            try:
                self.local_utils.exec_command('groupadd lpmac --system')
            except RbtConfigError as error:
                raise RbtConfigError(error.message())

    def __check_lpmac_group(self):
        """
        Проверяет наличие группы lpmac
        :return bool
        """
        return self.local_utils._check_call('cat /etc/group | grep lpmac')

    def __del_lpmac_group(self):
        """
        Удаляет группу lpmac
        :return None
        """
        self.local_utils.exec_command('groupdel lpmac')    

    def __remove_listen_rows(self):
        if not exists(self.__cups_d_conf):
            raise RbtConfigError('Файл \'%s\' не существует' % self.__cups_d_conf)
        self.config_parser.append_file_to_backup(self.__cups_d_conf)
        lines = open(self.__cups_d_conf, mode='r').readlines()
        results = []
        for line in lines:
            if not line.startswith('Listen'):
                results.append(line)
        open(self.__cups_d_conf, mode='w').writelines(results)
        
    def run_service(self, service):
        try:
            self.local_utils.exec_command('service %s start' % service)
        except RbtConfigError:
            return False
        return True
    
    def confirure_cupsctl_pam(self):
        parametrs = list()
        parametrs.append('--remote-admin')
        parametrs.append('--share-printers')
        parametrs.append('--remote-any')
        parametrs.append('ServerAlias=*')
        parametrs.append('DefaultAuthType=Basic')
        parametrs.append('DefaultPolicy=authenticated')
        self.local_utils.exec_command('cupsctl %s' % ' '.join(parametrs))  
        
    def reset_cupsctl_pam(self):
        parametrs = list()
        parametrs.append('--no-remote-admin')
        parametrs.append('--no-share-printers')
        parametrs.append('--no-remote-any')
        self.local_utils.exec_command('cupsctl %s' % ' '.join(parametrs))  
        
    def confirure_cupsctl_ald(self):
        parametrs = list()
        parametrs.append('--remote-admin')
        parametrs.append('--share-printers')
        parametrs.append('--remote-any')
        parametrs.append('ServerAlias=*')
        parametrs.append('DefaultPolicy=authenticated')
        parametrs.append('MarkerUser=ipp')
        parametrs.append('ServerName=%s' % self.ald_utils.hostname())
        parametrs.append('MacEnable=On')
        parametrs.append('DefaultAuthType=Negotiate')
        self.local_utils.exec_command('cupsctl %s' % ' '.join(parametrs))
        
    def reset_cupsctl_ald(self):
        parametrs = list()
        parametrs.append('--no-remote-admin')
        parametrs.append('--no-share-printers')
        parametrs.append('--no-remote-any')        
        self.local_utils.exec_command('cupsctl %s' % ' '.join(parametrs))          

    def configure_pam(self):
        self.run_service('cups') #TODO: потом поправить 
        self.exec_function_with_inverse_call(direct=self.confirure_cupsctl_pam,
                                             inverse=self.reset_cupsctl_pam)
        if not self.__check_lpmac_group():
            self.exec_function_with_inverse_call(direct=self.__add_lpmac_group,
                                                 inverse=self.__del_lpmac_group)
        self.__remove_listen_rows()
        self.config_parser.add_rows_to_config_file(self.__cups_d_conf, ['Listen 0.0.0.0:631'])
        self.config_parser.add_rows_to_config_file(self.__cups_d_conf, ['Listen /var/run/cups/cups.sock'])
        self.config_parser.add_rows_to_config_file(self.__cups_d_conf, ['ServerName %s' % BaseUtils.hostname()])

    def check_pam(self):
        checks = list()
        #TODO: Дописать проверку активности remote-admin и тд
        checks.append(self.__check_lpmac_group())
        checks.append(self.config_parser.config_file_contains_rows(self.__cups_d_conf,
                                                                   self.__cupd_d_listen_data_pam))
        return analyze_check(checks)
    
    def reset_pam(self):
        self.run_service('cups') #TODO: потом поправить         
        self.exec_function_with_inverse_call(direct=self.reset_cupsctl_pam,
                                             inverse=self.confirure_cupsctl_pam)
        
        self.config_parser.remove_rows_from_config(self.__cups_d_conf,
                                                   self.__cupd_d_listen_data_pam)        

    def configure_ald(self):
        self.config_parser.append_file_to_backup(self.__cups_d_conf)
        self.run_service('cups') #TODO: потом поправить
        self.run_service('krb5-admin-server') #TODO: потом поправить  
        if not self.ald_utils.check_is_service_in_domain():
            self.exec_function_with_inverse_call(direct=self.ald_utils.add_service_to_domain,
                                                 inverse=self.ald_utils.del_service_from_domain)
        if not self.ald_utils.check_is_service_in_group('mac'):
            self.exec_function_with_inverse_call(direct=self.ald_utils.add_service_to_group,
                                                 inverse=self.ald_utils.del_service_from_group,
                                                 direct_args=['mac'])
        if not self.ald_utils.check_is_exists_keytab():
            self.exec_function_with_inverse_call(direct=self.ald_utils.create_keytab_for_service,
                                                 inverse=self.ald_utils.remove_keytab)
        if not self.ald_utils.check_is_user_group_in_domain('lpmac'):          
            self.exec_function_with_inverse_call(direct=self.ald_utils.add_user_group_to_domain,
                                                 inverse=self.ald_utils.del_user_group_from_domain,
                                                 direct_args=['lpmac', 2900],
                                                 inverse_args=['lpmac'])        
        if not self.ald_utils.check_is_user_group_in_domain('print_admins'):
            self.exec_function_with_inverse_call(direct=self.ald_utils.add_user_group_to_domain,
                                                 inverse=self.ald_utils.del_user_group_from_domain,
                                                 direct_args=['print_admins', 3500],
                                                 inverse_args=['print_admins'])
        self.exec_function_with_inverse_call(direct=self.confirure_cupsctl_ald,
                                             inverse=self.reset_cupsctl_ald)         
        self.__remove_listen_rows()
        self.config_parser.remove_rows_from_config(self.__cups_d_conf, ['Port 631'])
        self.config_parser.add_rows_to_config_file(self.__cups_d_conf, ['Listen 0.0.0.0:631'])
        self.config_parser.add_rows_to_config_file(self.__cups_d_conf, ['Listen /var/run/cups/cups.sock'])
        self.config_parser.remove_rows_from_config(self.__cups_files_conf, ['SystemGroup lpadmin'])
        self.config_parser.add_rows_to_config_file(self.__cups_files_conf, ['SystemGroup print_admins'])

    def check_ald(self):
        checks = list()
        checks.append(self.ald_utils.check_is_exists_keytab())
        if self.ald_utils.check_if_ald_pass_file_exits():
            checks.append(self.ald_utils.check_is_service_in_domain())
            checks.append(self.ald_utils.check_is_user_group_in_domain('lpmac'))
            checks.append(self.ald_utils.check_is_service_in_group('mac'))
            checks.append(self.ald_utils.check_is_user_group_in_domain('print_admins'))
        checks.append(self.config_parser.config_file_contains_rows(self.__cups_d_conf, self.__cupd_d_listen_data_ald))
        checks.append(self.config_parser.config_file_contains_rows(self.__cups_files_conf,
                                                                   ['SystemGroup print_admins']))
        return analyze_check(checks)

    def reset_ald(self):
        self.run_service('cups') #TODO: потом поправить
        self.run_service('krb5-admin-server') #TODO: потом поправить  
        if self.ald_utils.check_is_exists_keytab():
            self.exec_function_with_inverse_call(direct=self.ald_utils.remove_keytab,
                                                 inverse=self.ald_utils.create_keytab_for_service)
        if self.ald_utils.check_is_service_in_group('mac'):
            self.exec_function_with_inverse_call(direct=self.ald_utils.del_service_from_group,
                                                 inverse=self.ald_utils.add_service_to_group,
                                                 direct_args=['mac'])
        if self.ald_utils.check_is_service_in_domain():
            self.exec_function_with_inverse_call(direct=self.ald_utils.del_service_from_domain,
                                                 inverse=self.ald_utils.add_service_to_domain)
        if self.ald_utils.check_is_user_group_in_domain('lpmac'):          
            self.exec_function_with_inverse_call(direct=self.ald_utils.del_user_group_from_domain,
                                                 inverse=self.ald_utils.add_user_group_to_domain,
                                                 direct_args=['lpmac'],
                                                 inverse_args=['lpmac', 2900])          
        if self.ald_utils.check_is_user_group_in_domain('print_admins'):
            self.exec_function_with_inverse_call(direct=self.ald_utils.del_user_group_from_domain,
                                                 inverse=self.ald_utils.add_user_group_to_domain,
                                                 direct_args=['print_admins'],
                                                 inverse_args=['print_admins', 3500])
        self.config_parser.remove_rows_from_config(self.__cups_d_conf, self.__cupd_d_listen_data_ald)
        self.config_parser.remove_rows_from_config(self.__cups_files_conf, ['SystemGroup print_admins'])
        self.exec_function_with_inverse_call(direct=self.reset_cupsctl_ald,
                                             inverse=self.confirure_cupsctl_ald) 

if __name__ == '__main__':
    run(Cups, param_required=False)
